Package: set6
Title: R6 Mathematical Sets Interface
Version: 0.2.3
Authors@R: 
      c(person(given = "Raphael",
             family = "Sonabend",
             role = c("aut","cre"),
             email = "raphaelsonabend@gmail.com",
             comment = c(ORCID = "0000-0001-9225-4654")),
      person(given = "Franz",
             family = "Kiraly",
             role = "aut",
             email = "f.kiraly@ucl.ac.uk"))
Description: An object-oriented package for mathematical sets, upgrading the current gold-standard {sets}. Many forms of mathematical sets are implemented, including (countably finite) sets, tuples, intervals (countably infinite or uncountable), and fuzzy variants. Wrappers extend functionality by allowing symbolic representations of complex operations on sets, including unions, (cartesian) products, exponentiation, and differences (asymmetric and symmetric).
LinkingTo: Rcpp
Imports: checkmate, Rcpp, R6, utils
Suggests: knitr, testthat, devtools, rmarkdown
License: MIT + file LICENSE
URL: https://xoopR.github.io/set6/, https://github.com/xoopR/set6
BugReports: https://github.com/xoopR/set6/issues
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.1.1
SystemRequirements: C++11
Collate: 'Properties.R' 'RcppExports.R' 'Set.R' 'SetWrapper.R'
        'SetWrapper_ComplementSet.R' 'SetWrapper_ExponentSet.R'
        'SetWrapper_PowersetSet.R' 'SetWrapper_ProductSet.R'
        'SetWrapper_UnionSet.R' 'Set_Complex.R' 'Set_ConditionalSet.R'
        'Set_FuzzySet.R' 'Set_FuzzySet_FuzzyMultiset.R'
        'Set_FuzzySet_FuzzyTuple.R' 'Set_Interval.R' 'setSymbol.R'
        'Set_Interval_SpecialSet.R' 'Set_LogicalSet.R' 'Set_Logicals.R'
        'Set_Multiset.R' 'Set_Tuple.R' 'Set_Universal.R'
        'Set_UniversalSet.R' 'asFuzzySet.R' 'asInterval.R' 'asSet.R'
        'helpers.R' 'assertions.R' 'atomic_coercions.R'
        'listSpecialSets.R' 'operation_cleaner.R'
        'operation_powerset.R' 'operation_setcomplement.R'
        'operation_setintersect.R' 'operation_setpower.R'
        'operation_setproduct.R' 'operation_setsymdiff.R'
        'operation_setunion.R' 'operators.R' 'set6-deprecated.R'
        'set6-package.R' 'set6.news.R' 'useUnicode.R' 'zzz.R'
NeedsCompilation: yes
Packaged: 2021-07-25 17:45:15 UTC; raphaelsonabend
Author: Raphael Sonabend [aut, cre] (<https://orcid.org/0000-0001-9225-4654>),
  Franz Kiraly [aut]
Maintainer: Raphael Sonabend <raphaelsonabend@gmail.com>
Repository: CRAN
Date/Publication: 2021-07-25 18:10:02 UTC
Built: R 4.0.5; x86_64-w64-mingw32; 2021-07-25 21:25:44 UTC; windows
Archs: x64
