# Generated by roxygen2: do not edit by hand

S3method("!=",Set)
S3method("&",Set)
S3method("*",Set)
S3method("+",Set)
S3method("-",Set)
S3method("<",Set)
S3method("<=",Set)
S3method("==",Set)
S3method(">",Set)
S3method(">=",Set)
S3method("^",Set)
S3method("|",Set)
S3method(as.FuzzyMultiset,ConditionalSet)
S3method(as.FuzzyMultiset,FuzzySet)
S3method(as.FuzzyMultiset,Interval)
S3method(as.FuzzyMultiset,Set)
S3method(as.FuzzyMultiset,data.frame)
S3method(as.FuzzyMultiset,list)
S3method(as.FuzzyMultiset,matrix)
S3method(as.FuzzyMultiset,numeric)
S3method(as.FuzzySet,ConditionalSet)
S3method(as.FuzzySet,FuzzySet)
S3method(as.FuzzySet,Interval)
S3method(as.FuzzySet,Set)
S3method(as.FuzzySet,data.frame)
S3method(as.FuzzySet,list)
S3method(as.FuzzySet,matrix)
S3method(as.FuzzySet,numeric)
S3method(as.FuzzyTuple,ConditionalSet)
S3method(as.FuzzyTuple,FuzzySet)
S3method(as.FuzzyTuple,Interval)
S3method(as.FuzzyTuple,Set)
S3method(as.FuzzyTuple,data.frame)
S3method(as.FuzzyTuple,list)
S3method(as.FuzzyTuple,matrix)
S3method(as.FuzzyTuple,numeric)
S3method(as.Interval,ConditionalSet)
S3method(as.Interval,Interval)
S3method(as.Interval,Set)
S3method(as.Interval,data.frame)
S3method(as.Interval,list)
S3method(as.Interval,matrix)
S3method(as.Interval,numeric)
S3method(as.Multiset,ConditionalSet)
S3method(as.Multiset,FuzzySet)
S3method(as.Multiset,Interval)
S3method(as.Multiset,Set)
S3method(as.Multiset,data.frame)
S3method(as.Multiset,default)
S3method(as.Multiset,list)
S3method(as.Multiset,matrix)
S3method(as.Multiset,numeric)
S3method(as.Set,ConditionalSet)
S3method(as.Set,FuzzySet)
S3method(as.Set,Interval)
S3method(as.Set,Set)
S3method(as.Set,data.frame)
S3method(as.Set,default)
S3method(as.Set,list)
S3method(as.Set,matrix)
S3method(as.Set,numeric)
S3method(as.Tuple,ConditionalSet)
S3method(as.Tuple,FuzzySet)
S3method(as.Tuple,Interval)
S3method(as.Tuple,Set)
S3method(as.Tuple,data.frame)
S3method(as.Tuple,default)
S3method(as.Tuple,list)
S3method(as.Tuple,matrix)
S3method(as.Tuple,numeric)
S3method(as.character,Set)
S3method(as.double,Set)
S3method(as.double,Tuple)
S3method(setcomplement,ComplementSet)
S3method(setcomplement,ConditionalSet)
S3method(setcomplement,FuzzySet)
S3method(setcomplement,Integers)
S3method(setcomplement,Interval)
S3method(setcomplement,Rationals)
S3method(setcomplement,Reals)
S3method(setcomplement,Set)
S3method(setintersect,ComplementSet)
S3method(setintersect,ConditionalSet)
S3method(setintersect,Interval)
S3method(setintersect,ProductSet)
S3method(setintersect,UnionSet)
S3method(summary,Set)
export("%-%")
export("%inset%")
export(ComplementSet)
export(Complex)
export(ConditionalSet)
export(ExponentSet)
export(ExtendedReals)
export(FuzzyMultiset)
export(FuzzySet)
export(FuzzyTuple)
export(Integers)
export(Interval)
export(LogicalSet)
export(Logicals)
export(Multiset)
export(Naturals)
export(NegIntegers)
export(NegRationals)
export(NegReals)
export(PosIntegers)
export(PosNaturals)
export(PosRationals)
export(PosReals)
export(PowersetSet)
export(ProductSet)
export(Rationals)
export(Reals)
export(Set)
export(Tuple)
export(UnionSet)
export(Universal)
export(UniversalSet)
export(as.FuzzyMultiset)
export(as.FuzzySet)
export(as.FuzzyTuple)
export(as.Interval)
export(as.Multiset)
export(as.Set)
export(as.Tuple)
export(assertClosed)
export(assertClosedAbove)
export(assertClosedBelow)
export(assertConditionalSet)
export(assertContains)
export(assertCountablyFinite)
export(assertCrisp)
export(assertEmpty)
export(assertFinite)
export(assertFuzzy)
export(assertFuzzyMultiset)
export(assertFuzzySet)
export(assertFuzzyTuple)
export(assertInterval)
export(assertMultiset)
export(assertSet)
export(assertSetList)
export(assertSubset)
export(assertTuple)
export(checkClosed)
export(checkClosedAbove)
export(checkClosedBelow)
export(checkConditionalSet)
export(checkContains)
export(checkCountablyFinite)
export(checkCrisp)
export(checkEmpty)
export(checkFinite)
export(checkFuzzy)
export(checkFuzzyMultiset)
export(checkFuzzySet)
export(checkFuzzyTuple)
export(checkInterval)
export(checkMultiset)
export(checkSet)
export(checkSetList)
export(checkSubset)
export(checkTuple)
export(listSpecialSets)
export(powerset)
export(set6News)
export(setcomplement)
export(setintersect)
export(setpower)
export(setproduct)
export(setsymdiff)
export(setunion)
export(testClosed)
export(testClosedAbove)
export(testClosedBelow)
export(testConditionalSet)
export(testContains)
export(testCountablyFinite)
export(testCrisp)
export(testEmpty)
export(testFinite)
export(testFuzzy)
export(testFuzzyMultiset)
export(testFuzzySet)
export(testFuzzyTuple)
export(testInterval)
export(testMultiset)
export(testSet)
export(testSetList)
export(testSubset)
export(testTuple)
export(useUnicode)
importFrom(R6,R6Class)
importFrom(Rcpp,sourceCpp)
useDynLib(set6, .registration = TRUE)
